@@ -3,6 +3,7 @@
  * All rights reserved.
  */
 
+#include <stdint.h>
 #include <stdio.h>
 
 #include "code.h"
@@ -15,6 +16,7 @@
 #include "parse.h"
 #include "resolve.h"
 #include "trace.h"
+#include "util.h"
 
 extern void	oj_set_obj_ivar(Val parent, Val kval, VALUE value);
 extern VALUE	oj_parse_xml_time(const char *str, int len); // from object.c
@@ -113,7 +115,7 @@ date_dump(VALUE obj, int depth, Out out) {
     } else {
 	volatile VALUE	v;
 	volatile VALUE	ov;
-	
+
 	switch (out->opts->time_format) {
 	case RubyTime:
 	case XmlTime:
@@ -148,7 +150,7 @@ date_dump(VALUE obj, int depth, Out out) {
 static VALUE
 date_load(VALUE clas, VALUE args) {
     volatile VALUE	v;
-    
+
     if (Qnil != (v = rb_hash_aref(args, rb_str_new2("s")))) {
 	return rb_funcall(oj_date_class, rb_intern("parse"), 1, v);
     }
@@ -158,7 +160,7 @@ date_load(VALUE clas, VALUE args) {
 static VALUE
 datetime_load(VALUE clas, VALUE args) {
     volatile VALUE	v;
-    
+
     if (Qnil != (v = rb_hash_aref(args, rb_str_new2("s")))) {
 	return rb_funcall(oj_datetime_class, rb_intern("parse"), 1, v);
     }
@@ -207,7 +209,7 @@ range_dump(VALUE obj, int depth, Out out) {
 static VALUE
 range_load(VALUE clas, VALUE args) {
     VALUE	nargs[3];
-    
+
     nargs[0] = rb_hash_aref(args, rb_id2str(oj_begin_id));
     nargs[1] = rb_hash_aref(args, rb_id2str(oj_end_id));
     nargs[2] = rb_hash_aref(args, rb_id2str(oj_exclude_end_id));
@@ -248,7 +250,7 @@ rational_load(VALUE clas, VALUE args) {
 static VALUE
 regexp_load(VALUE clas, VALUE args) {
     volatile VALUE	v;
-    
+
     if (Qnil != (v = rb_hash_aref(args, rb_str_new2("s")))) {
 	return rb_funcall(rb_cRegexp, oj_new_id, 1, v);
     }
@@ -421,7 +423,7 @@ dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
 	v = rb_funcall(obj, *odd->attrs, 0);
 	if (Qundef == v || T_STRING != rb_type(v)) {
 	    rb_raise(rb_eEncodingError, "Invalid type for raw JSON.\n");
-	} else {	    
+	} else {
 	    const char	*s = rb_string_value_ptr((VALUE*)&v);
 	    int		len = (int)RSTRING_LEN(v);
 	    const char	*name = rb_id2name(*odd->attrs);
@@ -457,7 +459,7 @@ dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
 		char	*n;
 		char	*end;
 		ID	i;
-	    
+
 		if (sizeof(nbuf) <= nlen) {
 		    if (NULL == (n2 = strdup(name))) {
 			rb_raise(rb_eNoMemError, "for attribute name.");
@@ -495,7 +497,10 @@ dump_odd(VALUE obj, Odd odd, VALUE clas, int depth, Out out) {
 // Return class if still needs dumping.
 static VALUE
 dump_common(VALUE obj, int depth, Out out) {
-    if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
+
+    if (Yes == out->opts->raw_json && rb_respond_to(obj, oj_raw_json_id)) {
+	oj_dump_raw_json(obj, depth, out);
+    } else if (Yes == out->opts->to_json && rb_respond_to(obj, oj_to_json_id)) {
 	volatile VALUE	rs;
 	const char	*s;
 	int		len;
@@ -606,7 +611,7 @@ dump_attr_cb(ID key, VALUE value, Out out) {
     oj_dump_custom_val(value, depth, out, true);
     out->depth = depth;
     *out->cur++ = ',';
-    
+
     return ST_CONTINUE;
 }
 
@@ -778,7 +783,7 @@ static void
 dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
     long	id = oj_check_circular(obj, out);
     VALUE	clas;
-    
+
     if (0 > id) {
 	oj_dump_nil(Qnil, depth, out, false);
     } else if (Qnil != (clas = dump_common(obj, depth, out))) {
@@ -791,7 +796,7 @@ dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
 	size_t		size = d2 * out->indent + d3 * out->indent + 3;
 	const char	*name;
 	int		cnt;
-	size_t		len;	
+	size_t		len;
 
 	assure_size(out, size);
 	if (clas == rb_cRange) {
@@ -1058,16 +1063,18 @@ hash_set_num(struct _parseInfo *pi, Val kval, NumInfo ni) {
 		// match the expected value.
 		parent->val = rb_funcall2(parent->val, oj_utc_id, 0, 0);
 	    } else if (ni->hasExp) {
-		time_t	t = (time_t)(ni->i + ni->exp);
-		struct tm	*st = gmtime(&t);
-		VALUE	args[8];
+		int64_t			t = (int64_t)(ni->i + ni->exp);
+		struct _timeInfo	ti;
+		VALUE			args[8];
 
-		args[0] = LONG2NUM(1900 + st->tm_year);
-		args[1] = LONG2NUM(1 + st->tm_mon);
-		args[2] = LONG2NUM(st->tm_mday);
-		args[3] = LONG2NUM(st->tm_hour);
-		args[4] = LONG2NUM(st->tm_min);
-		args[5] = rb_float_new((double)st->tm_sec + ((double)nsec + 0.5) / 1000000000.0);
+		sec_as_time(t, &ti);
+
+		args[0] = LONG2NUM(ti.year);
+		args[1] = LONG2NUM(ti.mon);
+		args[2] = LONG2NUM(ti.day);
+		args[3] = LONG2NUM(ti.hour);
+		args[4] = LONG2NUM(ti.min);
+		args[5] = rb_float_new((double)ti.sec + ((double)nsec + 0.5) / 1000000000.0);
 		args[6] = LONG2NUM(ni->exp);
 		parent->val = rb_funcall2(rb_cTime, oj_new_id, 7, args);
 	    } else {
@@ -1109,7 +1116,7 @@ static void
 array_append_num(ParseInfo pi, NumInfo ni) {
     Val			parent = stack_peek(&pi->stack);
     volatile VALUE	rval = oj_num_as_value(ni);
-    
+
     rb_ary_push(parent->val, rval);
     if (Yes == pi->options.trace) {
 	oj_trace_parse_call("append_number", pi, __FILE__, __LINE__, rval);
