@@ -3,22 +3,20 @@
  * All rights reserved.
  */
 
-#include <stdlib.h>
 #include <errno.h>
-#if !IS_WINDOWS
-#include <sys/time.h>
-#endif
-#include <time.h>
-#include <stdio.h>
-#include <string.h>
 #include <math.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
-#include <errno.h>
 
 #include "oj.h"
 #include "cache8.h"
 #include "dump.h"
 #include "odd.h"
+#include "trace.h"
+#include "util.h"
 
 // Workaround in case INFINITY is not defined in math.h or if the OS is CentOS
 #define OJ_INFINITY (1.0/0.0)
@@ -169,7 +167,7 @@ hixss_friendly_size(const uint8_t *str, size_t len) {
     size_t	size = 0;
     size_t	i = len;
     bool	check = false;
-    
+
     for (; 0 < i; str++, i--) {
 	size += hixss_friendly_chars[*str];
 	if (0 != (0x80 & *str)) {
@@ -204,7 +202,7 @@ rails_friendly_size(const uint8_t *str, size_t len) {
 const char*
 oj_nan_str(VALUE obj, int opt, int mode, bool plus, int *lenp) {
     const char	*str = NULL;
-    
+
     if (AutoNan == opt) {
 	switch (mode) {
 	case CompatMode:	opt = WordNan;	break;
@@ -326,7 +324,7 @@ dump_unicode(const char *str, const char *end, Out out, const char *orig) {
     *out->cur++ = 'u';
     for (i = 3; 0 <= i; i--) {
 	*out->cur++ = hex_chars[(uint8_t)(code >> (i * 4)) & 0x0F];
-    }	
+    }
     return str - 1;
 }
 
@@ -334,7 +332,7 @@ static const char*
 check_unicode(const char *str, const char *end, const char *orig) {
     uint8_t	b = *(uint8_t*)str;
     int		cnt = 0;
-    
+
     if (0xC0 == (0xE0 & b)) {
 	cnt = 1;
     } else if (0xE0 == (0xF0 & b)) {
@@ -414,7 +412,7 @@ oj_dump_time(VALUE obj, Out out, int withZone) {
     sec = rb_num2ll(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
     nsec = rb_num2ll(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
 #endif
-    
+
     *b-- = '\0';
     if (withZone) {
 	long	tzsecs = NUM2LONG(rb_funcall2(obj, oj_utc_offset_id, 0, 0));
@@ -493,14 +491,14 @@ oj_dump_ruby_time(VALUE obj, Out out) {
 
 void
 oj_dump_xml_time(VALUE obj, Out out) {
-    char	buf[64];
-    struct tm	*tm;
-    long	one = 1000000000;
-    time_t	sec;
-    long long	nsec;
-    long	tzsecs = NUM2LONG(rb_funcall2(obj, oj_utc_offset_id, 0, 0));
-    int		tzhour, tzmin;
-    char	tzsign = '+';
+    char		buf[64];
+    struct _timeInfo	ti;
+    long		one = 1000000000;
+    int64_t		sec;
+    long long		nsec;
+    long		tzsecs = NUM2LONG(rb_funcall2(obj, oj_utc_offset_id, 0, 0));
+    int			tzhour, tzmin;
+    char		tzsign = '+';
 
 #ifdef HAVE_RB_TIME_TIMESPEC
     if (16 <= sizeof(struct timespec)) {
@@ -541,8 +539,7 @@ oj_dump_xml_time(VALUE obj, Out out) {
     // 2012-01-05T23:58:07.123456000+09:00
     //tm = localtime(&sec);
     sec += tzsecs;
-    tm = gmtime(&sec);
-#if 1
+    sec_as_time((int64_t)sec, &ti);
     if (0 > tzsecs) {
         tzsign = '-';
         tzhour = (int)(tzsecs / -3600);
@@ -551,26 +548,12 @@ oj_dump_xml_time(VALUE obj, Out out) {
         tzhour = (int)(tzsecs / 3600);
         tzmin = (int)(tzsecs / 60) - (tzhour * 60);
     }
-#else
-    if (0 > tm->tm_gmtoff) {
-        tzsign = '-';
-        tzhour = (int)(tm->tm_gmtoff / -3600);
-        tzmin = (int)(tm->tm_gmtoff / -60) - (tzhour * 60);
-    } else {
-        tzhour = (int)(tm->tm_gmtoff / 3600);
-        tzmin = (int)(tm->tm_gmtoff / 60) - (tzhour * 60);
-    }
-#endif
     if (0 == nsec || 0 == out->opts->sec_prec) {
 	if (0 == tzsecs && rb_funcall2(obj, oj_utcq_id, 0, 0)) {
-	    sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02dZ",
-		    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		    tm->tm_hour, tm->tm_min, tm->tm_sec);
+	    sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02dZ", ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec);
 	    oj_dump_cstr(buf, 20, 0, 0, out);
 	} else {
-	    sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02d%c%02d:%02d",
-		    tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		    tm->tm_hour, tm->tm_min, tm->tm_sec,
+	    sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02d%c%02d:%02d", ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec,
 		    tzsign, tzhour, tzmin);
 	    oj_dump_cstr(buf, 25, 0, 0, out);
 	}
@@ -582,9 +565,7 @@ oj_dump_xml_time(VALUE obj, Out out) {
 	    format[32] = '0' + out->opts->sec_prec;
 	    len -= 9 - out->opts->sec_prec;
 	}
-	sprintf(buf, format,
-		tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		tm->tm_hour, tm->tm_min, tm->tm_sec, (long)nsec);
+	sprintf(buf, format, ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec, (long)nsec);
 	oj_dump_cstr(buf, len, 0, 0, out);
     } else {
 	char	format[64] = "%04d-%02d-%02dT%02d:%02d:%02d.%09ld%c%02d:%02d";
@@ -594,10 +575,7 @@ oj_dump_xml_time(VALUE obj, Out out) {
 	    format[32] = '0' + out->opts->sec_prec;
 	    len -= 9 - out->opts->sec_prec;
 	}
-	sprintf(buf, format,
-		tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		tm->tm_hour, tm->tm_min, tm->tm_sec, (long)nsec,
-		tzsign, tzhour, tzmin);
+	sprintf(buf, format, ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec, (long)nsec, tzsign, tzhour, tzmin);
 	oj_dump_cstr(buf, len, 0, 0, out);
     }
 }
@@ -763,6 +741,30 @@ debug_raise(const char *orig, size_t cnt, int line) {
     rb_raise(oj_json_generator_error_class, "Partial character in string. %s @ %d", buf, line);
 }
 
+void
+oj_dump_raw_json(VALUE obj, int depth, Out out) {
+    if (oj_string_writer_class == rb_obj_class(obj)) {
+	StrWriter	sw = (StrWriter)DATA_PTR(obj);
+	size_t		len = sw->out.cur - sw->out.buf;
+
+	if (0 < len) {
+	    len--;
+	}
+	oj_dump_raw(sw->out.buf, len, out);
+    } else {
+	volatile VALUE	jv;
+
+	if (Yes == out->opts->trace) {
+	    oj_trace("raw_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyIn);
+	}
+	jv = rb_funcall(obj, oj_raw_json_id, 2, RB_INT2NUM(depth), RB_INT2NUM(out->indent));
+	if (Yes == out->opts->trace) {
+	    oj_trace("raw_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyOut);
+	}
+	oj_dump_raw(rb_string_value_ptr((VALUE*)&jv), (size_t)RSTRING_LEN(jv), out);
+    }
+}
+
 void
 oj_dump_cstr(const char *str, size_t cnt, bool is_sym, bool escape1, Out out) {
     size_t	size;
@@ -824,7 +826,7 @@ oj_dump_cstr(const char *str, size_t cnt, bool is_sym, bool escape1, Out out) {
     } else {
 	const char	*end = str + cnt;
 	const char	*check_start = str;
-	
+
 	if (is_sym) {
 	    *out->cur++ = ':';
 	}
@@ -890,13 +892,13 @@ oj_dump_cstr(const char *str, size_t cnt, bool is_sym, bool escape1, Out out) {
 		break; // ignore, should never happen if the table is correct
 	    }
 	}
-	*out->cur++ = '"'; 
+	*out->cur++ = '"';
     }
     if ((JXEsc == out->opts->escape_mode || RailsXEsc == out->opts->escape_mode) && 0 < str - orig && 0 != (0x80 & *(str - 1))) {
 	uint8_t	c = (uint8_t)*(str - 1);
 	int	i;
 	int	scnt = (int)(str - orig);
-	
+
 	// Last utf-8 characters must be 0x10xxxxxx. The start must be
 	// 0x110xxxxx for 2 characters, 0x1110xxxx for 3, and 0x11110xxx for
 	// 4.
@@ -962,7 +964,7 @@ oj_grow_out(Out out, size_t len) {
     size_t  size = out->end - out->buf;
     long    pos = out->cur - out->buf;
     char    *buf = out->buf;
-	
+
     size *= 2;
     if (size <= len * 2 + pos) {
 	size += len;
@@ -1019,24 +1021,21 @@ oj_dump_fixnum(VALUE obj, int depth, Out out, bool as_ok) {
     char	*b = buf + sizeof(buf) - 1;
     long long	num = rb_num2ll(obj);
     int		neg = 0;
-	bool	dump_as_string = false;
+    bool	dump_as_string = false;
 
-	if (out->opts->integer_range_max != 0 && out->opts->integer_range_min != 0 &&
-		(out->opts->integer_range_max < num || out->opts->integer_range_min > num)) {
+    if (out->opts->integer_range_max != 0 && out->opts->integer_range_min != 0 &&
+	(out->opts->integer_range_max < num || out->opts->integer_range_min > num)) {
 	dump_as_string = true;
-	}
-
+    }
     if (0 > num) {
 	neg = 1;
 	num = -num;
     }
-
     *b-- = '\0';
 
-	if (dump_as_string) {
+    if (dump_as_string) {
 	*b-- = '"';
-	}
-
+    }
     if (0 < num) {
 	for (; 0 < num; num /= 10, b--) {
 	    *b = (num % 10) + '0';
@@ -1049,11 +1048,9 @@ oj_dump_fixnum(VALUE obj, int depth, Out out, bool as_ok) {
     } else {
 	*b = '0';
     }
-
-	if (dump_as_string) {
+    if (dump_as_string) {
 	*--b = '"';
-	}
-
+    }
     assure_size(out, (sizeof(buf) - (b - buf)));
     for (; '\0' != *b; b++) {
 	*out->cur++ = *b;
@@ -1068,7 +1065,7 @@ oj_dump_bignum(VALUE obj, int depth, Out out, bool as_ok) {
 	bool		dump_as_string = false;
 
 	if (out->opts->integer_range_max != 0 || out->opts->integer_range_min != 0) { // Bignum cannot be inside of Fixnum range
-	dump_as_string = true; 
+	dump_as_string = true;
 	assure_size(out, cnt + 2);
 	*out->cur++ = '"';
 	} else {
@@ -1106,7 +1103,7 @@ oj_dump_float(VALUE obj, int depth, Out out, bool as_ok) {
 	    cnt = sizeof(inf_val) - 1;
 	} else {
 	    NanDump	nd = out->opts->dump_opts.nan_dump;
-	    
+
 	    if (AutoNan == nd) {
 		switch (out->opts->mode) {
 		case CompatMode:	nd = WordNan;	break;
@@ -1141,7 +1138,7 @@ oj_dump_float(VALUE obj, int depth, Out out, bool as_ok) {
 	    cnt = sizeof(ninf_val) - 1;
 	} else {
 	    NanDump	nd = out->opts->dump_opts.nan_dump;
-	    
+
 	    if (AutoNan == nd) {
 		switch (out->opts->mode) {
 		case CompatMode:	nd = WordNan;	break;
@@ -1175,7 +1172,7 @@ oj_dump_float(VALUE obj, int depth, Out out, bool as_ok) {
 	    cnt = sizeof(ninf_val) - 1;
 	} else {
 	    NanDump	nd = out->opts->dump_opts.nan_dump;
-	    
+
 	    if (AutoNan == nd) {
 		switch (out->opts->mode) {
 		case ObjectMode:	nd = HugeNan;	break;
