@@ -53,6 +53,7 @@ ID	oj_length_id;
 ID	oj_new_id;
 ID	oj_parse_id;
 ID	oj_pos_id;
+ID	oj_raw_json_id;
 ID	oj_read_id;
 ID	oj_readpartial_id;
 ID	oj_replace_id;
@@ -94,6 +95,7 @@ VALUE	oj_hash_class_sym;
 VALUE	oj_indent_sym;
 VALUE	oj_object_class_sym;
 VALUE	oj_quirks_mode_sym;
+VALUE	oj_safe_sym;
 VALUE	oj_trace_sym;
 
 static VALUE	allow_blank_sym;
@@ -137,6 +139,7 @@ static VALUE	unicode_xss_sym;
 static VALUE	unix_sym;
 static VALUE	unix_zone_sym;
 static VALUE	use_as_json_sym;
+static VALUE	use_raw_json_sym;
 static VALUE	use_to_hash_sym;
 static VALUE	use_to_json_sym;
 static VALUE	wab_sym;
@@ -168,6 +171,7 @@ struct _options	oj_default_options = {
     No,		// to_hash
     No,		// to_json
     No,		// as_json
+    No,		// raw_json
     No,		// nilnil
     Yes,	// empty_string
     Yes,	// allow_gc
@@ -176,6 +180,7 @@ struct _options	oj_default_options = {
     No,		// create_ok
     Yes,	// allow_nan
     No,		// trace
+    No,		// safe
     0,		// integer_range_min
     0,		// integer_range_max
     oj_json_class,	// create_id
@@ -229,6 +234,7 @@ struct _options	oj_default_options = {
  * - *:float_precision* [_Fixnum_|_nil_] number of digits of precision when dumping floats, 0 indicates use Ruby
  * - *:use_to_json* [_Boolean_|_nil_] call to_json() methods on dump, default is false
  * - *:use_as_json* [_Boolean_|_nil_] call as_json() methods on dump, default is false
+ * - *:use_raw_json* [_Boolean_|_nil_] call raw_json() methods on dump, default is false
  * - *:nilnil* [_Boolean_|_nil_] if true a nil input to load will return nil and not raise an Exception
  * - *:empty_string* [_Boolean_|_nil_] if true an empty input will not raise an Exception
  * - *:allow_gc* [_Boolean_|_nil_] allow or prohibit GC during parsing, default is true (allow)
@@ -245,8 +251,9 @@ struct _options	oj_default_options = {
  * - *:array_class* [_Class_|_nil_] Class to use instead of Array on load
  * - *:omit_nil* [_true_|_false_] if true Hash and Object attributes with nil values are omitted
  * - *:ignore* [_nil_|Array] either nil or an Array of classes to ignore when dumping
- * - *:integer_range* [_Range_] Dump integers outside range as strings. 
+ * - *:integer_range* [_Range_] Dump integers outside range as strings.
  * - *:trace* [_true,_|_false_] Trace all load and dump calls, default is false (trace is off)
+ * - *:safe* [_true,_|_false_] Safe mimic breaks JSON mimic to be safer, default is false (safe is off)
  *
  * Return [_Hash_] all current option settings.
  */
@@ -269,6 +276,7 @@ get_def_opts(VALUE self) {
     rb_hash_aset(opts, use_to_json_sym, (Yes == oj_default_options.to_json) ? Qtrue : ((No == oj_default_options.to_json) ? Qfalse : Qnil));
     rb_hash_aset(opts, use_to_hash_sym, (Yes == oj_default_options.to_hash) ? Qtrue : ((No == oj_default_options.to_hash) ? Qfalse : Qnil));
     rb_hash_aset(opts, use_as_json_sym, (Yes == oj_default_options.as_json) ? Qtrue : ((No == oj_default_options.as_json) ? Qfalse : Qnil));
+    rb_hash_aset(opts, use_raw_json_sym, (Yes == oj_default_options.raw_json) ? Qtrue : ((No == oj_default_options.raw_json) ? Qfalse : Qnil));
     rb_hash_aset(opts, nilnil_sym, (Yes == oj_default_options.nilnil) ? Qtrue : ((No == oj_default_options.nilnil) ? Qfalse : Qnil));
     rb_hash_aset(opts, empty_string_sym, (Yes == oj_default_options.empty_string) ? Qtrue : ((No == oj_default_options.empty_string) ? Qfalse : Qnil));
     rb_hash_aset(opts, allow_gc_sym, (Yes == oj_default_options.allow_gc) ? Qtrue : ((No == oj_default_options.allow_gc) ? Qfalse : Qnil));
@@ -276,6 +284,7 @@ get_def_opts(VALUE self) {
     rb_hash_aset(opts, allow_invalid_unicode_sym, (Yes == oj_default_options.allow_invalid) ? Qtrue : ((No == oj_default_options.allow_invalid) ? Qfalse : Qnil));
     rb_hash_aset(opts, oj_allow_nan_sym, (Yes == oj_default_options.allow_nan) ? Qtrue : ((No == oj_default_options.allow_nan) ? Qfalse : Qnil));
     rb_hash_aset(opts, oj_trace_sym, (Yes == oj_default_options.trace) ? Qtrue : ((No == oj_default_options.trace) ? Qfalse : Qnil));
+    rb_hash_aset(opts, oj_safe_sym, (Yes == oj_default_options.safe) ? Qtrue : ((No == oj_default_options.safe) ? Qfalse : Qnil));
     rb_hash_aset(opts, float_prec_sym, INT2FIX(oj_default_options.float_prec));
     switch (oj_default_options.mode) {
     case StrictMode:	rb_hash_aset(opts, mode_sym, strict_sym);	break;
@@ -287,7 +296,7 @@ get_def_opts(VALUE self) {
     case WabMode:	rb_hash_aset(opts, mode_sym, wab_sym);		break;
     default:		rb_hash_aset(opts, mode_sym, object_sym);	break;
     }
-    
+
     if (oj_default_options.integer_range_max != 0 || oj_default_options.integer_range_min != 0) {
     VALUE range = rb_obj_alloc(rb_cRange);
     VALUE min = LONG2FIX(oj_default_options.integer_range_min);
@@ -343,7 +352,7 @@ get_def_opts(VALUE self) {
     } else {
 	VALUE		*vp;
 	volatile VALUE	a = rb_ary_new();
-	
+
 	for (vp = oj_default_options.ignore; Qnil != *vp; vp++) {
 	    rb_ary_push(a, *vp);
 	}
@@ -374,6 +383,7 @@ get_def_opts(VALUE self) {
  *   - *:use_to_json* [_Boolean_|_nil_] call to_json() methods on dump, default is false.
  *   - *:use_as_json* [_Boolean_|_nil_] call as_json() methods on dump, default is false.
  *   - *:use_to_hash* [_Boolean_|_nil_] call to_hash() methods on dump, default is false.
+ *   - *:use_raw_json* [_Boolean_|_nil_] call raw_json() methods on dump, default is false.
  *   - *:nilnil* [_Boolean_|_nil_] if true a nil input to load will return nil and not raise an Exception.
  *   - *:allow_gc* [_Boolean_|_nil_] allow or prohibit GC during parsing, default is true (allow).
  *   - *:quirks_mode* [_Boolean_|_nil_] allow single JSON values instead of documents, default is true (allow).
@@ -388,8 +398,9 @@ get_def_opts(VALUE self) {
  *   - *:array_class* [_Class_|_nil_] Class to use instead of Array on load.
  *   - *:omit_nil* [_true_|_false_] if true Hash and Object attributes with nil values are omitted.
  *   - *:ignore* [_nil_|Array] either nil or an Array of classes to ignore when dumping
- *   - *:integer_range* [_Range_] Dump integers outside range as strings. 
+ *   - *:integer_range* [_Range_] Dump integers outside range as strings.
  *   - *:trace* [_Boolean_] turn trace on or off.
+ *   - *:safe* [_Boolean_] turn safe mimic on or off.
  */
 static VALUE
 set_def_opts(VALUE self, VALUE opts) {
@@ -410,6 +421,7 @@ oj_parse_options(VALUE ropts, Options copts) {
 	{ use_to_hash_sym, &copts->to_hash },
 	{ use_to_json_sym, &copts->to_json },
 	{ use_as_json_sym, &copts->as_json },
+	{ use_raw_json_sym, &copts->raw_json },
 	{ nilnil_sym, &copts->nilnil },
 	{ allow_blank_sym, &copts->nilnil }, // same as nilnil
 	{ empty_string_sym, &copts->empty_string },
@@ -418,13 +430,14 @@ oj_parse_options(VALUE ropts, Options copts) {
 	{ allow_invalid_unicode_sym, &copts->allow_invalid },
 	{ oj_allow_nan_sym, &copts->allow_nan },
 	{ oj_trace_sym, &copts->trace },
+	{ oj_safe_sym, &copts->safe },
 	{ oj_create_additions_sym, &copts->create_ok },
 	{ Qnil, 0 }
     };
     YesNoOpt		o;
     volatile VALUE	v;
     size_t		len;
-    
+
     if (T_HASH != rb_type(ropts)) {
 	return;
     }
@@ -715,7 +728,7 @@ oj_parse_options(VALUE ropts, Options copts) {
 	    cnt = (int)RARRAY_LEN(v);
 	    if (0 < cnt) {
 		int	i;
-		
+
 		copts->ignore = ALLOC_N(VALUE, cnt + 1);
 		for (i = 0; i < cnt; i++) {
 		    copts->ignore[i] = rb_ary_entry(v, i);
@@ -802,11 +815,10 @@ oj_parse_opt_match_string(RxClass rc, VALUE ropts) {
  *
  * - *json* [_String_|_IO_] JSON String or an Object that responds to read()
  * - *options* [_Hash_] load options (same as default_options)
- *   - -
  * - *obj* [_Hash_|_Array_|_String_|_Fixnum_|_Float_|_Boolean_|_nil_] parsed object.
  * - *start* [_optional, _Integer_] start position of parsed JSON for obj.
  * - *len* [_optional, _Integer_] length of parsed JSON for obj.
- * 
+ *
  * Returns [_Hash_|_Array_|_String_|_Fixnum_|_Float_|_Boolean_|_nil_]
  */
 static VALUE
@@ -889,7 +901,6 @@ load(int argc, VALUE *argv, VALUE self) {
  *
  * - *path* [_String_] to a file containing a JSON document
  * - *options* [_Hash_] load options (same as default_options)
- *   - -
  * - *obj* [_Hash_|_Array_|_String_|_Fixnum_|_Float_|_Boolean_|_nil_] parsed object.
  * - *start* [_optional, _Integer_] start position of parsed JSON for obj.
  * - *len* [_optional, _Integer_] length of parsed JSON for obj.
@@ -1064,7 +1075,7 @@ dump(int argc, VALUE *argv, VALUE self) {
  * Dumps an Object (obj) to a string. If the object has a to_json method that
  * will be called. The mode is set to :compat.
  * - *obj* [_Object_] Object to serialize as an JSON document String
- * - *options* [_Hash_] 
+ * - *options* [_Hash_]
  *   - *:max_nesting* [_boolean_] It true nesting is limited to 100. The option to detect circular references is available but is not compatible with the json gem., default is false
  *   - *:allow_nan* [_boolean_] If true non JSON compliant words such as Nan and Infinity will be used as appropriate, default is true.
  *   - *:quirks_mode* [_boolean_] Allow single JSON values instead of documents, default is true (allow).
@@ -1126,7 +1137,7 @@ to_json(int argc, VALUE *argv, VALUE self) {
 static VALUE
 to_file(int argc, VALUE *argv, VALUE self) {
     struct _options	copts = oj_default_options;
-    
+
     if (3 == argc) {
 	oj_parse_options(argv[2], &copts);
     }
@@ -1149,7 +1160,7 @@ to_file(int argc, VALUE *argv, VALUE self) {
 static VALUE
 to_stream(int argc, VALUE *argv, VALUE self) {
     struct _options	copts = oj_default_options;
-    
+
     if (3 == argc) {
 	oj_parse_options(argv[2], &copts);
     }
@@ -1265,7 +1276,6 @@ register_odd_raw(int argc, VALUE *argv, VALUE self) {
  *
  * - *json* [_String_|_IO_] JSON String or an Object that responds to read().
  * - *options* [_Hash_] load options (same as default_options).
- *   - -
  * - *obj* [_Hash_|_Array_|_String_|_Fixnum_|_Float_|_Boolean_|_nil_] parsed object.
  * - *start* [_optional, _Integer_] start position of parsed JSON for obj.
  * - *len* [_optional, _Integer_] length of parsed JSON for obj.
@@ -1300,7 +1310,6 @@ extern VALUE	oj_strict_parse(int argc, VALUE *argv, VALUE self);
  *
  * - *json* [_String_|_IO_] JSON String or an Object that responds to read().
  * - *options* [_Hash_] load options (same as default_options).
- *   - -
  * - *obj* [_Hash_|_Array_|_String_|_Fixnum_|_Float_|_Boolean_|_nil_] parsed object.
  * - *start* [_optional, _Integer_] start position of parsed JSON for obj.
  * - *len* [_optional, _Integer_] length of parsed JSON for obj.
@@ -1322,22 +1331,6 @@ extern VALUE	oj_compat_parse(int argc, VALUE *argv, VALUE self);
  * valid. If the input is not a valid JSON document (an empty string is not a
  * valid JSON document) an exception is raised.
  *
- * Note: Oj is not able to automatically deserialize all classes that are a
- * subclass of a Ruby Exception. Only exception that take one required string
- * argument in the initialize() method are supported. This is an example of how
- * to write an Exception subclass that supports both a single string intializer
- * and an Exception as an argument. Additional optional arguments can be added
- * as well.
- *
- * The reason for this restriction has to do with a design decision on the part
- * of the Ruby developers. Exceptions are special Objects. They do not follow the
- * rules of other Objects. Exceptions have 'mesg' and a 'bt' attribute. Note that
- * these are not '@mesg' and '@bt'. They can not be set using the normal C or
- * Ruby calls. The only way I have found to set the 'mesg' attribute is through
- * the initializer. Unfortunately that means any subclass that provides a
- * different initializer can not be automatically decoded. A way around this is
- * to use a create function but this example shows an alternative.
- *
  * A block can be provided with a single argument. That argument will be the
  * parsed JSON document. This is useful when parsing a string that includes
  * multiple JSON documents. The block can take up to 3 arguments, the parsed
@@ -1347,7 +1340,6 @@ extern VALUE	oj_compat_parse(int argc, VALUE *argv, VALUE self);
  *
  * - *json* [_String_|_IO_] JSON String or an Object that responds to read().
  * - *options* [_Hash_] load options (same as default_options).
- *   - -
  * - *obj* [_Hash_|_Array_|_String_|_Fixnum_|_Float_|_Boolean_|_nil_] parsed object.
  * - *start* [_optional, _Integer_] start position of parsed JSON for obj.
  * - *len* [_optional, _Integer_] length of parsed JSON for obj.
@@ -1383,7 +1375,6 @@ extern VALUE	oj_object_parse(int argc, VALUE *argv, VALUE self);
  *
  * - *json* [_String_|_IO_] JSON String or an Object that responds to read().
  * - *options* [_Hash_] load options (same as default_options).
- *   - -
  * - *obj* [_Hash_|_Array_|_String_|_Fixnum_|_Float_|_Boolean_|_nil_] parsed object.
  * - *start* [_optional, _Integer_] start position of parsed JSON for obj.
  * - *len* [_optional, _Integer_] length of parsed JSON for obj.
@@ -1445,16 +1436,16 @@ extern VALUE	oj_define_mimic_json(int argc, VALUE *argv, VALUE self);
 
 /* Document-method: generate
  * call-seq: generate(obj, opts=nil)
- * 
+ *
  * Encode obj as a JSON String. The obj argument must be a Hash, Array, or
  * respond to to_h or to_json. Options other than those listed such as
  * +:allow_nan+ or +:max_nesting+ are ignored.
- * 
+ *
  * - *obj* [_Object__|_Hash_|_Array_] object to convert to a JSON String
  * - *opts* [_Hash_] options
- * - - *:indent* [_String_] String to use for indentation.
+ *   - *:indent* [_String_] String to use for indentation.
  *   - *:space* [_String_] String placed after a , or : delimiter
- *   - *:space * _before [_String_] String placed before a : delimiter
+ *   - *:space_before* [_String_] String placed before a : delimiter
  *   - *:object_nl* [_String_] String placed after a JSON object
  *   - *:array_nl* [_String_] String placed after a JSON array
  *   - *:ascii_only* [_Boolean_] if not nil or false then use only ascii characters in the output. Note JSON.generate does support this even if it is not documented.
@@ -1497,15 +1488,15 @@ protect_require(VALUE x) {
  * modes are:
  *
  * - *:strict* mode will only allow the 7 basic JSON types to be serialized. Any other Object
- *   will raise an Exception. 
- * 
+ *   will raise an Exception.
+ *
  * - *:null* mode is similar to the :strict mode except any Object that is not
  *   one of the JSON base types is replaced by a JSON null.
- * 
+ *
  * - *:object* mode will dump any Object as a JSON Object with keys that match
  *   the Ruby Object's variable names without the '@' character. This is the
  *   highest performance mode.
- * 
+ *
  * - *:compat* or *:json* mode is the compatible mode for the json gem. It mimics
  *   the json gem including the options, defaults, and restrictions.
  *
@@ -1591,6 +1582,7 @@ Init_oj() {
     oj_new_id = rb_intern("new");
     oj_parse_id = rb_intern("parse");
     oj_pos_id = rb_intern("pos");
+    oj_raw_json_id = rb_intern("raw_json");
     oj_read_id = rb_intern("read");
     oj_readpartial_id = rb_intern("readpartial");
     oj_replace_id = rb_intern("replace");
@@ -1667,6 +1659,7 @@ Init_oj() {
     oj_object_class_sym = ID2SYM(rb_intern("object_class"));	rb_gc_register_address(&oj_object_class_sym);
     oj_object_nl_sym = ID2SYM(rb_intern("object_nl"));		rb_gc_register_address(&oj_object_nl_sym);
     oj_quirks_mode_sym = ID2SYM(rb_intern("quirks_mode"));	rb_gc_register_address(&oj_quirks_mode_sym);
+    oj_safe_sym = ID2SYM(rb_intern("safe"));			rb_gc_register_address(&oj_safe_sym);
     oj_space_before_sym = ID2SYM(rb_intern("space_before"));	rb_gc_register_address(&oj_space_before_sym);
     oj_space_sym = ID2SYM(rb_intern("space"));			rb_gc_register_address(&oj_space_sym);
     oj_trace_sym = ID2SYM(rb_intern("trace"));			rb_gc_register_address(&oj_trace_sym);
@@ -1682,6 +1675,7 @@ Init_oj() {
     unix_sym = ID2SYM(rb_intern("unix"));			rb_gc_register_address(&unix_sym);
     unix_zone_sym = ID2SYM(rb_intern("unix_zone"));		rb_gc_register_address(&unix_zone_sym);
     use_as_json_sym = ID2SYM(rb_intern("use_as_json"));		rb_gc_register_address(&use_as_json_sym);
+    use_raw_json_sym = ID2SYM(rb_intern("use_raw_json"));	rb_gc_register_address(&use_raw_json_sym);
     use_to_hash_sym = ID2SYM(rb_intern("use_to_hash"));		rb_gc_register_address(&use_to_hash_sym);
     use_to_json_sym = ID2SYM(rb_intern("use_to_json"));		rb_gc_register_address(&use_to_json_sym);
     wab_sym = ID2SYM(rb_intern("wab"));				rb_gc_register_address(&wab_sym);
