@@ -139,6 +139,7 @@ typedef struct _options {
     char		to_hash;	// YesNo
     char		to_json;	// YesNo
     char		as_json;	// YesNo
+    char		raw_json;	// YesNo
     char		nilnil;		// YesNo
     char		empty_string;	// YesNo
     char		allow_gc;	// allow GC during parse
@@ -147,6 +148,7 @@ typedef struct _options {
     char		create_ok;	// YesNo allow create_id
     char		allow_nan;	// YEsyNo for parsing only
     char		trace;		// YesNo
+    char		safe;		// YesNo
     int64_t		integer_range_min; // dump numbers outside range as string
     int64_t		integer_range_max;
     const char		*create_id;	// 0 or string
@@ -202,7 +204,7 @@ typedef struct _strWriter {
     char		*types;	// DumpType
     char		*types_end;
     int			keyWritten;
-    
+
 } *StrWriter;
 
 typedef struct _streamWriter {
@@ -219,7 +221,7 @@ enum {
     COL_VAL  = 0x02,
     RUBY_VAL = 0x03
 };
-    
+
 typedef struct _leaf {
     struct _leaf	*next;
     union {
@@ -349,6 +351,7 @@ extern ID	oj_readpartial_id;
 extern ID	oj_replace_id;
 extern ID	oj_stat_id;
 extern ID	oj_string_id;
+extern ID	oj_raw_json_id;
 extern ID	oj_to_h_id;
 extern ID	oj_to_hash_id;
 extern ID	oj_to_json_id;
@@ -365,6 +368,7 @@ extern ID	oj_write_id;
 
 extern bool	oj_use_hash_alt;
 extern bool	oj_use_array_alt;
+extern bool	string_writer_optimized;
 
 #if HAVE_LIBPTHREAD
 extern pthread_mutex_t	oj_cache_mutex;
