@@ -8,6 +8,7 @@
 #include "code.h"
 #include "encode.h"
 #include "trace.h"
+#include "util.h"
 
 #define OJ_INFINITY (1.0/0.0)
 
@@ -116,7 +117,7 @@ dump_attr_cb(ID key, VALUE value, Out out) {
     dump_rails_val(value, depth, out, true);
     out->depth = depth;
     *out->cur++ = ',';
-    
+
     return ST_CONTINUE;
 }
 
@@ -221,15 +222,15 @@ dump_bigdecimal(VALUE obj, int depth, Out out, bool as_ok) {
 }
 
 static void
-dump_sec_nano(VALUE obj, time_t sec, long nsec, Out out) {
+dump_sec_nano(VALUE obj, int64_t sec, long nsec, Out out) {
     char		buf[64];
-    struct tm		*tm;
+    struct _timeInfo	ti;
     long		one = 1000000000;
     long		tzsecs = NUM2LONG(rb_funcall2(obj, oj_utc_offset_id, 0, 0));
     int			tzhour, tzmin;
     char		tzsign = '+';
     int			len;
-    
+
     if (out->end - out->cur <= 36) {
 	assure_size(out, 36);
     }
@@ -248,7 +249,7 @@ dump_sec_nano(VALUE obj, time_t sec, long nsec, Out out) {
     }
     // 2012-01-05T23:58:07.123456000+09:00 or 2012/01/05 23:58:07 +0900
     sec += tzsecs;
-    tm = gmtime(&sec);
+    sec_as_time(sec, &ti);
     if (0 > tzsecs) {
         tzsign = '-';
         tzhour = (int)(tzsecs / -3600);
@@ -258,19 +259,12 @@ dump_sec_nano(VALUE obj, time_t sec, long nsec, Out out) {
         tzmin = (int)(tzsecs / 60) - (tzhour * 60);
     }
     if (!xml_time) {
-	len = sprintf(buf, "%04d/%02d/%02d %02d:%02d:%02d %c%02d%02d",
-		      tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		      tm->tm_hour, tm->tm_min, tm->tm_sec, tzsign, tzhour, tzmin);
+	len = sprintf(buf, "%04d/%02d/%02d %02d:%02d:%02d %c%02d%02d", ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec, tzsign, tzhour, tzmin);
     } else if (0 == out->opts->sec_prec) {
 	if (0 == tzsecs && rb_funcall2(obj, oj_utcq_id, 0, 0)) {
-	    len = sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02dZ",
-			  tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-			  tm->tm_hour, tm->tm_min, tm->tm_sec);
+	    len = sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02dZ", ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec);
 	} else {
-	    len = sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02d%c%02d:%02d",
-			  tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-			  tm->tm_hour, tm->tm_min, tm->tm_sec,
-			  tzsign, tzhour, tzmin);
+	    len = sprintf(buf, "%04d-%02d-%02dT%02d:%02d:%02d%c%02d:%02d", ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec, tzsign, tzhour, tzmin);
 	}
     } else if (0 == tzsecs && rb_funcall2(obj, oj_utcq_id, 0, 0)) {
 	char	format[64] = "%04d-%02d-%02dT%02d:%02d:%02d.%09ldZ";
@@ -280,9 +274,7 @@ dump_sec_nano(VALUE obj, time_t sec, long nsec, Out out) {
 	    format[32] = '0' + out->opts->sec_prec;
 	    len -= 9 - out->opts->sec_prec;
 	}
-	len = sprintf(buf, format,
-		      tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		      tm->tm_hour, tm->tm_min, tm->tm_sec, nsec);
+	len = sprintf(buf, format, ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec, nsec);
     } else {
 	char	format[64] = "%04d-%02d-%02dT%02d:%02d:%02d.%09ld%c%02d:%02d";
 
@@ -291,10 +283,7 @@ dump_sec_nano(VALUE obj, time_t sec, long nsec, Out out) {
 	    format[32] = '0' + out->opts->sec_prec;
 	    len -= 9 - out->opts->sec_prec;
 	}
-	len = sprintf(buf, format,
-		      tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday,
-		      tm->tm_hour, tm->tm_min, tm->tm_sec, nsec,
-		      tzsign, tzhour, tzmin);
+	len = sprintf(buf, format, ti.year, ti.mon, ti.day, ti.hour, ti.min, ti.sec, nsec, tzsign, tzhour, tzmin);
     }
     oj_dump_cstr(buf, len, 0, 0, out);
 }
@@ -323,7 +312,7 @@ dump_time(VALUE obj, int depth, Out out, bool as_ok) {
 
 static void
 dump_timewithzone(VALUE obj, int depth, Out out, bool as_ok) {
-    time_t	sec = NUM2LONG(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
+    int64_t	sec = NUM2LONG(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
     long long	nsec = 0;
 
     if (rb_respond_to(obj, oj_tv_nsec_id)) {
@@ -364,7 +353,7 @@ columns_array(VALUE rcols, int *ccnt) {
     StrLen		cols;
     int			i;
     int			cnt = (int)RARRAY_LEN(rcols);
-    
+
     *ccnt = cnt;
     cols = ALLOC_N(struct _strLen, cnt);
     for (i = 0, cp = cols; i < cnt; i++, cp++) {
@@ -383,7 +372,7 @@ dump_row(VALUE row, StrLen cols, int ccnt, int depth, Out out) {
     size_t	size;
     int		d2 = depth + 1;
     int		i;
-				   
+
     assure_size(out, 2);
     *out->cur++ = '{';
     size = depth * out->indent + 3;
@@ -443,7 +432,7 @@ dump_activerecord_result(VALUE obj, int depth, Out out, bool as_ok) {
     int			i, rcnt;
     size_t		size;
     int			d2 = depth + 1;
-    
+
     if (0 == rows_id) {
 	rows_id = rb_intern("@rows");
 	columns_id = rb_intern("@columns");
@@ -608,7 +597,7 @@ create_opt(ROptTable rot, VALUE clas) {
 	ro = &rot->table[olen];
     } else {
 	int	i;
-	
+
 	for (i = 0, ro = rot->table; i < olen; i++, ro++) {
 	    if (clas < ro->clas) {
 		memmove(ro + 1, ro, sizeof(struct _rOpt) * (olen - i));
@@ -644,7 +633,7 @@ create_opt(ROptTable rot, VALUE clas) {
 	    ro->dump = dump_to_s;
 	}
     }
-    return NULL;
+    return ro;
 }
 
 static void
@@ -683,7 +672,7 @@ encoder_new(int argc, VALUE *argv, VALUE self) {
     e->opts = oj_default_options;
     e->arg = Qnil;
     copy_opts(&ropts, &e->ropts);
-    
+
     if (1 <= argc && Qnil != *argv) {
 	oj_parse_options(*argv, &e->opts);
 	e->arg = *argv;
@@ -738,7 +727,7 @@ optimize(int argc, VALUE *argv, ROptTable rot, bool on) {
 	int		i;
 	NamedFunc	nf;
 	VALUE		clas;
-	
+
 	oj_rails_hash_opt = on;
 	oj_rails_array_opt = on;
 	oj_rails_float_opt = on;
@@ -761,6 +750,8 @@ optimize(int argc, VALUE *argv, ROptTable rot, bool on) {
 	    oj_rails_array_opt = on;
 	} else if (rb_cFloat == *argv) {
 	    oj_rails_float_opt = on;
+	} else if (oj_string_writer_class == *argv) {
+	    string_writer_optimized = on;
 	} else if (NULL != (ro = oj_rails_get_opt(rot, *argv)) ||
 		   NULL != (ro = create_opt(rot, *argv))) {
 	    ro->on = on;
@@ -770,14 +761,14 @@ optimize(int argc, VALUE *argv, ROptTable rot, bool on) {
 
 /* Document-method optimize
  *	call-seq: optimize(*classes)
- * 
+ *
  * Use Oj rails optimized routines to encode the specified classes. This
  * ignores the as_json() method on the class and uses an internal encoding
  * instead. Passing in no classes indicates all should use the optimized
  * version of encoding for all previously optimized classes. Passing in the
  * Object class set a global switch that will then use the optimized behavior
  * for all classes.
- * 
+ *
  * - *classes* [_Class_] a list of classes to optimize
  */
 static VALUE
@@ -791,19 +782,20 @@ encoder_optimize(int argc, VALUE *argv, VALUE self) {
 
 /* Document-method: optimize
  *	call-seq: optimize(*classes)
- * 
+ *
  * Use Oj rails optimized routines to encode the specified classes. This
  * ignores the as_json() method on the class and uses an internal encoding
  * instead. Passing in no classes indicates all should use the optimized
  * version of encoding for all previously optimized classes. Passing in the
  * Object class set a global switch that will then use the optimized behavior
  * for all classes.
- * 
+ *
  * - *classes* [_Class_] a list of classes to optimize
  */
 static VALUE
 rails_optimize(int argc, VALUE *argv, VALUE self) {
     optimize(argc, argv, &ropts, true);
+    string_writer_optimized = true;
 
     return Qnil;
 }
@@ -817,7 +809,7 @@ rails_optimize(int argc, VALUE *argv, VALUE self) {
 VALUE
 rails_mimic_json(VALUE self) {
     VALUE	json;
-    
+
     if (rb_const_defined_at(rb_cObject, rb_intern("JSON"))) {
 	json = rb_const_get_at(rb_cObject, rb_intern("JSON"));
     } else {
@@ -831,7 +823,7 @@ rails_mimic_json(VALUE self) {
 
 /* Document-method: deoptimize
  *	call-seq: deoptimize(*classes)
- * 
+ *
  * Turn off Oj rails optimization on the specified classes.
  *
  * - *classes* [_Class_] a list of classes to deoptimize
@@ -847,7 +839,7 @@ encoder_deoptimize(int argc, VALUE *argv, VALUE self) {
 
 /* Document-method: deoptimize
  *	call-seq: deoptimize(*classes)
- * 
+ *
  * Turn off Oj rails optimization on the specified classes.
  *
  * - *classes* [_Class_] a list of classes to deoptimize
@@ -855,13 +847,14 @@ encoder_deoptimize(int argc, VALUE *argv, VALUE self) {
 static VALUE
 rails_deoptimize(int argc, VALUE *argv, VALUE self) {
     optimize(argc, argv, &ropts, false);
+    string_writer_optimized = false;
 
     return Qnil;
 }
 
 /* Document-method:optimized?
  *	call-seq: optimized?(clas)
- * 
+ *
  * - *clas* [_Class_] Class to check
  *
  * @return true if the class is being optimized for rails and false otherwise
@@ -879,7 +872,7 @@ encoder_optimized(VALUE self, VALUE clas) {
 
 /* Document-method: optimized?
  *	call-seq: optimized?(clas)
- * 
+ *
  * Returns true if the specified Class is being optimized.
  */
 static VALUE
@@ -977,7 +970,7 @@ encode(VALUE obj, ROptTable ropts, Options opts, int argc, VALUE *argv) {
 
 /* Document-method: encode
  *	call-seq: encode(obj)
- * 
+ *
  * - *obj* [_Object_] object to encode
  *
  * Returns encoded object as a JSON string.
@@ -988,7 +981,7 @@ encoder_encode(VALUE self, VALUE obj) {
 
     if (Qnil != e->arg) {
 	VALUE	argv[1] = { e->arg };
-	
+
 	return encode(obj, &e->ropts, &e->opts, 1, argv);
     }
     return encode(obj, &e->ropts, &e->opts, 0, NULL);
@@ -996,9 +989,9 @@ encoder_encode(VALUE self, VALUE obj) {
 
 /* Document-method: encode
  *	call-seq: encode(obj, opts=nil)
- * 
+ *
  * Encode obj as a JSON String.
- * 
+ *
  * - *obj* [_Object_|Hash|Array] object to convert to a JSON String
  * - *opts* [_Hash_] options
  *
@@ -1048,7 +1041,7 @@ rails_time_precision(VALUE self, VALUE prec) {
 
 /* Document-method: set_encoder
  *	call-seq: set_encoder()
- * 
+ *
  * Sets the ActiveSupport.encoder to Oj::Rails::Encoder and wraps some of the
  * formatting globals used by ActiveSupport to allow the use of those globals
  * in the Oj::Rails optimizations.
@@ -1060,7 +1053,7 @@ rails_set_encoder(VALUE self) {
     VALUE	encoding;
     VALUE	pv;
     VALUE	verbose;
-    
+
     if (rb_const_defined_at(rb_cObject, rb_intern("ActiveSupport"))) {
 	active = rb_const_get_at(rb_cObject, rb_intern("ActiveSupport"));
     } else {
@@ -1101,7 +1094,7 @@ rails_set_decoder(VALUE self) {
     VALUE	json;
     VALUE	json_error;
     VALUE	verbose;
-    
+
     if (rb_const_defined_at(rb_cObject, rb_intern("JSON"))) {
 	json = rb_const_get_at(rb_cObject, rb_intern("JSON"));
     } else {
@@ -1124,7 +1117,7 @@ rails_set_decoder(VALUE self) {
     rb_undef_method(json, "parse");
     rb_define_module_function(json, "parse", oj_mimic_parse, -1);
     rb_gv_set("$VERBOSE", verbose);
-    
+
     return Qnil;
 }
 
@@ -1144,7 +1137,7 @@ oj_optimize_rails(VALUE self) {
 }
 
 /* Document-module: Oj::Rails
- * 
+ *
  * Module that provides rails and active support compatibility.
  */
 /* Document-class: Oj::Rails::Encoder
@@ -1154,7 +1147,7 @@ oj_optimize_rails(VALUE self) {
 void
 oj_mimic_rails_init() {
     VALUE	rails = rb_define_module_under(Oj, "Rails");
-    
+
     rb_define_module_function(rails, "encode", rails_encode, -1);
 
     encoder_class = rb_define_class_under(rails, "Encoder", rb_cObject);
@@ -1294,7 +1287,10 @@ hash_cb(VALUE key, VALUE value, Out out) {
     int		depth = out->depth;
     long	size;
     int		rtype = rb_type(key);
-    
+
+    if (out->omit_nil && Qnil == value) {
+	return ST_CONTINUE;
+    }
     if (rtype != T_STRING && rtype != T_SYMBOL) {
 	key = rb_funcall(key, oj_to_s_id, 0);
 	rtype = rb_type(key);
@@ -1409,6 +1405,8 @@ dump_obj(VALUE obj, int depth, Out out, bool as_ok) {
 
 	if (NULL != (ro = oj_rails_get_opt(out->ropts, rb_obj_class(obj))) && ro->on) {
 	    ro->dump(obj, depth, out, as_ok);
+	} else if (Yes == out->opts->raw_json && rb_respond_to(obj, oj_raw_json_id)) {
+	    oj_dump_raw_json(obj, depth, out);
 	} else if (rb_respond_to(obj, oj_as_json_id)) {
 	    dump_as_json(obj, depth, out, true);
 	} else if (rb_respond_to(obj, oj_to_hash_id)) {
@@ -1416,6 +1414,8 @@ dump_obj(VALUE obj, int depth, Out out, bool as_ok) {
 	} else {
 	    oj_dump_obj_to_s(obj, out);
 	}
+    } else if (Yes == out->opts->raw_json && rb_respond_to(obj, oj_raw_json_id)) {
+	oj_dump_raw_json(obj, depth, out);
     } else if (rb_respond_to(obj, oj_to_hash_id)) {
 	// Always attempt to_hash.
 	dump_to_hash(obj, depth, out);
