@@ -134,6 +134,7 @@ class Juice < Minitest::Test
       :use_to_json=>false,
       :use_to_hash=>false,
       :use_as_json=>false,
+      :use_raw_json=>false,
       :nilnil=>true,
       :empty_string=>true,
       :allow_gc=>false,
@@ -158,6 +159,7 @@ class Juice < Minitest::Test
       :array_class=>Array,
       :ignore=>nil,
       :trace=>true,
+      :safe=>true,
     }
     Oj.default_options = alt
     #keys = alt.keys
@@ -420,6 +422,14 @@ class Juice < Minitest::Test
     assert_equal('-2208987661.000000000', json)
   end
 
+  def test_time_years
+    (-2020..2020).each { |year|
+      s = "%04d-03-01T15:14:13Z" % [year]
+      json = Oj.dump(Time.parse(s), mode: :custom, time_format: :xmlschema)
+      assert_equal(s, json[1..-2])
+    }
+  end
+
   # Class
   def test_class_null
     json = Oj.dump(Juice, :mode => :null)
@@ -687,8 +697,8 @@ class Juice < Minitest::Test
 
   def test_bad_bignum
     if '2.4.0' < RUBY_VERSION
-      assert_raises Exception do # Can be either Oj::ParseError or ArgumentError depending on Ruby version
-	Oj.load(%|{ "big": -e123456789 }|)
+      assert_raises Oj::ParseError do
+	Oj.load(%|{ "big": -e123456789 }|, mode: :strict)
       end
     end
   end
